part of 'collection_value.dart';

class SetValue<E> extends _CollectionValue<Set<E>>
    with
        _ImmutableIterableMixin<E, Set<E>>,
        _ImmutableSetMixin<E>,
        _MutableWithThrowSetMixin<E> {
  SetValue(Iterable<E> value)
      : super(
          value?.any((e) => e == null) ?? true
              ? _throwNullError()
              : Set.of(value),
          _CollectionEquality(SetEquality<E>()),
        );

  @override
  int get hashCode => _equality.getHashCode(this);

  @override
  bool operator ==(Object other) =>
      other is SetValue<E> && _equality.equals(this, other);
}

mixin _ImmutableSetMixin<E> on _ImmutableIterableMixin<E, Set<E>>
    implements Set<E> {
  @override
  Set<R> cast<R>() => _base.cast();

  @override
  bool containsAll(Iterable<Object> other) => _base.containsAll(other);

  @override
  Set<E> difference(Set<Object> other) => _base.difference(other);

  @override
  Set<E> intersection(Set<Object> other) => _base.intersection(other);

  @override
  E lookup(Object object) => _base.lookup(object);

  @override
  Set<E> union(Set<E> other) => _base.union(other);
}

mixin _MutableWithThrowSetMixin<E> on _ImmutableSetMixin<E> {
  @alwaysThrows
  R _throw<R>() {
    throw UnsupportedError('Cannot modify an immutable Set<$E>');
  }

  @override
  bool add(E value) => _throw();

  @override
  void addAll(Iterable<E> elements) => _throw();

  @override
  void clear() => _throw();

  @override
  bool remove(Object value) => _throw();

  @override
  void removeAll(Iterable<Object> elements) => _throw();

  @override
  void removeWhere(bool Function(E element) test) => _throw();

  @override
  void retainAll(Iterable<Object> elements) => _throw();

  @override
  void retainWhere(bool Function(E element) test) => _throw();
}
